@@ -7,11 +7,11 @@ use Carp qw(croak);
 use MozRepl::Plugin::JSON2;
 
 use vars qw[$VERSION];
-$VERSION = '0.24';
+$VERSION = '0.25';
 
 =head1 NAME
 
-AnyEvent-enabled MozRepl client
+MozRepl::AnyEvent - AnyEvent-enabled MozRepl client
 
 =head1 SYNOPSIS
 
@@ -25,6 +25,17 @@ provide plugin support. If you want a fully compatible
 AnyEvent-enabled L<MozRepl>, please consider porting L<Net::Telnet>
 to L<AnyEvent::Handle>.
 
+Instead of using the process environment, you can also pass
+the backend class using the C<repl_class> parameter in the constructor:
+
+  use MozRepl::RemoteObject;
+  my $bridge = MozRepl::RemoteObject->install_bridge(
+      repl_class => 'MozRepl::AnyEvent',
+  );
+
+The module in C<repl_class> will be loaded through C<require>. If the module
+does not load, the fatal error will propagate. No fallbacks are tried.
+
 =head1 METHODS
 
 =head2 C<< MozRepl::AnyEvent->new( %options ) >>
