@@ -3,7 +3,7 @@ use strict;
 use Scalar::Util qw(blessed);
 
 use vars qw[$VERSION];
-$VERSION = '0.24';
+$VERSION = '0.25';
 
 =head1 NAME
 
@@ -48,13 +48,13 @@ sub invoke {
     die unless $self->__id;
     
     ($fn) = $self->MozRepl::RemoteObject::Methods::transform_arguments($fn);
-    my $rn = $self->bridge->name;
+    my $rn = bridge($self)->name;
     @args = $self->MozRepl::RemoteObject::Methods::transform_arguments(@args);
     local $" = ',';
     my $js = <<JS;
 $rn.callMethod($id,$fn,[@args])
 JS
-    return $self->bridge->unjson($js);
+    return bridge($self)->unjson($js);
 }
 
 =head2 C<< $obj->MozRepl::RemoteObject::Methods::transform_arguments(@args) >>
@@ -76,7 +76,7 @@ as a number through to Javascript, or to pass digits as a Javascript string.
  
 sub transform_arguments {
     my $self = shift;
-    my $json = $self->bridge->json;
+    my $json = bridge($self)->json;
     map {
         if (! defined) {
              'null'
@@ -84,13 +84,13 @@ sub transform_arguments {
             $_
         #} elsif (ref and blessed $_ and $_->isa(__PACKAGE__)) {
         } elsif (ref and blessed $_ and $_->isa('MozRepl::RemoteObject::Instance')) {
-            sprintf "%s.getLink(%d)", $_->bridge->name, $_->__id
+            sprintf "%s.getLink(%d)", bridge($_)->name, id($_)
         } elsif (ref and blessed $_ and $_->isa('MozRepl::RemoteObject')) {
             $_->name
         } elsif (ref and ref eq 'CODE') { # callback
             my $cb = $self->bridge->make_callback($_);
-            sprintf "%s.getLink(%d)", $self->bridge->name,
-                                      $cb->__id
+            sprintf "%s.getLink(%d)", bridge($self)->name,
+                                      id($cb)
         } elsif (ref) {
             $json->encode($_);
         } else {
@@ -149,6 +149,70 @@ Perl object.
 
 sub bridge { hash_get( $_[0], 'bridge' )};
 
+=head2 C<< MozRepl::RemoteObject::Methods::as_hash($obj) >>
+
+=head2 C<< MozRepl::RemoteObject::Methods::as_array($obj) >>
+
+=head2 C<< MozRepl::RemoteObject::Methods::as_code($obj) >>
+
+Returns a reference to a hash/array/coderef. This is used
+by L<overload>. Don't use these directly.
+
+=cut
+
+
+sub as_hash {
+    my $self = shift;
+    tie my %h, 'MozRepl::RemoteObject::TiedHash', $self;
+    \%h;
+};
+
+sub as_array {
+    my $self = shift;
+    tie my @a, 'MozRepl::RemoteObject::TiedArray', $self;
+    \@a;
+};
+
+sub as_code {
+    my $self = shift;
+    my $class = ref $self;
+    my $id = id($self);
+    my $context = hash_get($self, 'return_context');
+    $context = $context
+               ? qq{,"$context"}
+               : '';
+    return sub {
+        my (@args) = @_;
+        my $bridge = bridge($self);
+        
+        my $rn = $bridge->name;
+        @args = transform_arguments($self,@args);
+        local $" = ',';
+        my $js = <<JS;
+    $rn.callThis($id,[@args]$context)
+JS
+        return $bridge->unjson($js);
+    };
+};
+
+sub object_identity {
+    my ($self,$other) = @_;
+    return if (   ! $other 
+               or ! ref $other
+               or ! blessed $other
+               or ! $other->isa('MozRepl::RemoteObject::Instance')
+               or ! $self->isa('MozRepl::RemoteObject::Instance'));
+    my $left = id($self)
+        or die "Internal inconsistency - no id found for $self";
+    my $right = id($other);
+    my $bridge = bridge($self);
+    my $rn = $bridge->name;
+    my $data = $bridge->js_call_to_perl_struct(<<JS);
+    // __object_identity
+$rn.getLink($left)===$rn.getLink($right)
+JS
+}
+
 1;
 
 __END__
